@@ -104,8 +104,11 @@ def cxx_loop(visit):
 
         # handle the body of the for loop
         orelse = [self.visit(stmt) for stmt in node.orelse]
-        orelse_label = Label(break_handler)
-        return Block([res] + orelse + [orelse_label])
+        if break_handler in self.used_break:
+            orelse_label = [Label(break_handler)]
+        else:
+            orelse_label = []
+        return Block([res] + orelse + orelse_label)
     return loop_visitor
 
 
@@ -193,6 +196,7 @@ class CxxFunction(ast.NodeVisitor):
         """ Basic initialiser gathering analysis informations. """
         self.parent = parent
         self.break_handlers = []
+        self.used_break = set()
         self.ldecls = None
         self.openmp_deps = set()
 
@@ -483,10 +487,14 @@ class CxxFunction(ast.NodeVisitor):
         local_target = "__target{0}".format(id(node))
         local_target_decl = self.types.builder.IteratorOfType(local_iter_decl)
 
+        islocal = (node.target.id not in self.openmp_deps and
+                   node.target.id in self.scope[node] and
+                   not hasattr(self, 'yields'))
         # If variable is local to the for body it's a ref to the iterator value
         # type
-        if node.target.id in self.scope[node] and not hasattr(self, 'yields'):
+        if islocal:
             local_type = "auto&&"
+            self.ldecls.remove(node.target.id)
         else:
             local_type = ""
 
@@ -567,9 +575,13 @@ class CxxFunction(ast.NodeVisitor):
         else:
             upper_bound = "__target{0}".format(id(node))
 
+        islocal = (node.target.id not in self.openmp_deps and
+                   node.target.id in self.scope[node] and
+                   not hasattr(self, 'yields'))
         # If variable is local to the for body keep it local...
-        if node.target.id in self.scope[node] and not hasattr(self, 'yields'):
+        if islocal:
             loop = list()
+            self.ldecls.remove(node.target.id)
         else:
             # For yield function, upper_bound is globals.
             iter_type = ""
@@ -645,7 +657,6 @@ class CxxFunction(ast.NodeVisitor):
                     node.target.id in self.scope[node] and
                     node.target.id not in self.openmp_deps)
         auto_for &= not metadata.get(node, OMPDirective)
-        auto_for &= node.target.id not in self.openmp_deps
         return auto_for
 
     def can_use_c_for(self, node):
@@ -827,6 +838,7 @@ class CxxFunction(ast.NodeVisitor):
         See Also : cxx_loop
         """
         if self.break_handlers and self.break_handlers[-1]:
+            self.used_break.add(self.break_handlers[-1])
             return Statement("goto {0}".format(self.break_handlers[-1]))
         else:
             return Statement("break")
@@ -877,11 +889,13 @@ class CxxFunction(ast.NodeVisitor):
 
             # constructor disambiguation, clang++ workaround
             if len(elts) == 1:
-                elts.append('pythonic::types::single_value()')
-
-            return "{0}({{{1}}})".format(
-                self.types.builder.Assignable(node_type).generate(self.lctx),
-                ", ".join(elts))
+                return "{0}({1}, pythonic::types::single_value())".format(
+                    self.types.builder.Assignable(node_type).generate(self.lctx),
+                    elts[0])
+            else:
+                return "{0}({{{1}}})".format(
+                    self.types.builder.Assignable(node_type).generate(self.lctx),
+                    ", ".join(elts))
 
     def visit_Set(self, node):
         if not node.elts:  # empty set
@@ -889,10 +903,17 @@ class CxxFunction(ast.NodeVisitor):
         else:
             elts = [self.visit(n) for n in node.elts]
             node_type = self.types.builder.Assignable(self.types[node])
-            return "{0}{{{{{1}}}}}".format(
-                node_type,
-                ", ".join("static_cast<{}::value_type>({})"
-                          .format(node_type, elt) for elt in elts))
+
+            # constructor disambiguation, clang++ workaround
+            if len(elts) == 1:
+                return "{0}({1}, pythonic::types::single_value())".format(
+                    self.types.builder.Assignable(node_type).generate(self.lctx),
+                    elts[0])
+            else:
+                return "{0}{{{{{1}}}}}".format(
+                    node_type,
+                    ", ".join("static_cast<{}::value_type>({})"
+                              .format(node_type, elt) for elt in elts))
 
     def visit_Dict(self, node):
         if not node.keys:  # empty dict
@@ -1048,6 +1069,7 @@ class CxxGenerator(CxxFunction):
 
     # stmt
     def visit_FunctionDef(self, node):
+        self.returns = False
         tmp = self.prepare_functiondef_context(node)
         operator_body, formal_types, formal_args = tmp
 
@@ -1060,8 +1082,9 @@ class CxxGenerator(CxxFunction):
             next_name,
             "<{0}>".format(", ".join(formal_types)) if formal_types else "")
 
-        operator_body.append(Statement("{}: return result_type()".format(
-            CxxGenerator.FinalStatement)))
+        if self.returns:
+            operator_body.append(Label(CxxGenerator.FinalStatement))
+        operator_body.append(Statement("return result_type()"))
 
         next_declaration = [
             FunctionDeclaration(Value("result_type", "next"), []),
@@ -1211,6 +1234,7 @@ class CxxGenerator(CxxFunction):
         return [next_struct, topstruct], [next_definition, operator_definition]
 
     def visit_Return(self, node):
+        self.returns = True
         return Block([Statement("{0} = -1".format(CxxGenerator.StateHolder)),
                       Statement("goto {0}".format(CxxGenerator.FinalStatement))
                       ])
